import { ByteArray } from '@regax/protocol';
import { EventEmitter, PlainData } from '@regax/common';
import { RegaxWebSocket, WebSocketClass, HandshakeBuffer } from './socket';
export interface RSA {
    generate(num: number, type: string): void;
    signString(str: string, type: string): any;
    n: any;
    e: any;
}
export interface ClientOpts {
    url?: string;
    encrypt?: boolean;
    rsa?: RSA;
    user?: PlainData;
    reconnect?: boolean;
    maxReconnectAttempts?: number;
    WebSocket?: WebSocketClass;
}
export interface Response {
    code?: number | string;
    message?: string;
    data?: PlainData;
}
export declare type Route = number | string;
export declare class Client extends EventEmitter<string> {
    protected reqId: number;
    protected callbacks: {
        [key: string]: {
            res: (d?: PlainData) => void;
            rej: (e: Error) => void;
        };
    };
    protected routeMap: {
        [key: string]: string | number;
    };
    protected dict: {
        [key: string]: string | number;
    };
    protected socket: RegaxWebSocket;
    protected handshakeBuffer: HandshakeBuffer;
    protected useCrypto: boolean;
    protected rsa: RSA;
    protected opts: ClientOpts;
    constructor(opts?: ClientOpts);
    protected decode(data: ByteArray): any;
    protected encode(id: number, route: string | number, data: any): ByteArray;
    protected sendMessage(id: number, route: Route, data: any): void;
    protected onMessage(data: ByteArray): void;
    connect(opts?: ClientOpts): Promise<HandshakeBuffer>;
    disconnect(): void;
    request(route: Route, msg?: {}): Promise<any>;
    notify(route: Route, msg?: {}): void;
    config(opts: ClientOpts): void;
}
//# sourceMappingURL=client.d.ts.map