import { EventEmitter, PlainData } from '@regax/common';
import { ByteArray, PackageDataType } from '@regax/protocol';
export declare enum SocketEvent {
    CONNECTION = "connection",
    RECONNECT = "reconnect",
    DISCONNECT = "disconnect",
    HEARTBEAT = "heartbeat",
    ERROR = "error",
    DATA = "data",
    KICK = "kick"
}
export declare const DEFAULT_MAX_RECONNECT_ATTEMPTS = 10;
export declare const DEFAULT_RECONNECT_DELAY: number;
export interface HandshakeBuffer {
    sys: {
        type: string;
        version: string;
        rsa?: {
            rsa_n?: string | number;
            rsa_e?: string | number;
        };
    };
    user: PlainData;
}
export interface WebSocket {
    binaryType: string;
    onclose: ((ev: any) => any) | null;
    onerror: ((ev: any) => any) | null;
    onmessage: ((ev: any) => any) | null;
    onopen: ((ev: any) => any) | null;
    close(code?: number, reason?: string): void;
    send(data: string | ArrayBufferLike | Blob | ArrayBufferView): void;
}
export interface WebSocketClass {
    prototype: WebSocket;
    new (url: string, protocols?: string | string[]): WebSocket;
}
export interface RegaxWebSocketConnectOpts {
    reconnect?: boolean;
    maxReconnectAttempts?: number;
    handshakeBuffer: HandshakeBuffer;
    WebSocket?: WebSocketClass;
}
export declare class RegaxWebSocket extends EventEmitter<SocketEvent> {
    static event: typeof SocketEvent;
    private socket?;
    private reconnect;
    private reconncetTimer;
    private reconnectAttempts;
    private reconnectionDelay;
    private maxReconnectAttempts;
    private handshakeBuffer;
    private heartbeat;
    protected isClosed: boolean;
    readonly event: typeof SocketEvent;
    connect(url: string, opts: RegaxWebSocketConnectOpts): this;
    protected tryToReconnectIfFailed(url: string, opts: RegaxWebSocketConnectOpts): void;
    send(buffer: ByteArray): this;
    close(): void;
    protected onMessage(msg: PackageDataType): void;
    protected reconnectReset(): void;
}
//# sourceMappingURL=socket.d.ts.map