"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable:no-any
var common_1 = require("@regax/common");
var protocol_1 = require("@regax/protocol");
var SocketEvent;
(function (SocketEvent) {
    SocketEvent["CONNECTION"] = "connection";
    SocketEvent["RECONNECT"] = "reconnect";
    SocketEvent["DISCONNECT"] = "disconnect";
    SocketEvent["HEARTBEAT"] = "heartbeat";
    SocketEvent["ERROR"] = "error";
    SocketEvent["DATA"] = "data";
    SocketEvent["KICK"] = "kick"; // kick user
})(SocketEvent = exports.SocketEvent || (exports.SocketEvent = {}));
var ResCode;
(function (ResCode) {
    ResCode[ResCode["OK"] = 200] = "OK";
    ResCode[ResCode["FAIL"] = 500] = "FAIL";
    ResCode[ResCode["OLD_CLIENT"] = 501] = "OLD_CLIENT";
})(ResCode || (ResCode = {}));
exports.DEFAULT_MAX_RECONNECT_ATTEMPTS = 10;
exports.DEFAULT_RECONNECT_DELAY = 5 * 1000;
var RegaxWebSocket = /** @class */ (function (_super) {
    __extends(RegaxWebSocket, _super);
    function RegaxWebSocket() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.reconnectAttempts = 0;
        _this.reconnectionDelay = exports.DEFAULT_RECONNECT_DELAY;
        _this.heartbeat = new common_1.Tick();
        _this.isClosed = false;
        _this.event = SocketEvent;
        return _this;
    }
    RegaxWebSocket.prototype.connect = function (url, opts) {
        var _this = this;
        if (this.socket)
            this.close();
        this.reconnect = opts.reconnect || false;
        this.handshakeBuffer = opts.handshakeBuffer;
        this.maxReconnectAttempts = opts.maxReconnectAttempts || exports.DEFAULT_MAX_RECONNECT_ATTEMPTS;
        var socket = this.socket = opts.WebSocket ? new opts.WebSocket(url) : new WebSocket(url);
        socket.binaryType = 'arraybuffer';
        socket.onopen = function () {
            if (_this.reconnect) {
                _this.emit(SocketEvent.RECONNECT);
            }
            _this.reconnectReset();
            // send handshake buffer when connected
            var packet = protocol_1.Package.encode(protocol_1.PackageType.HANDSHAKE, protocol_1.strencode(JSON.stringify(_this.handshakeBuffer)));
            _this.send(packet);
        };
        socket.onmessage = function (event) {
            var msgs = protocol_1.Package.decode(event.data);
            if (Array.isArray(msgs)) {
                for (var i = 0; i < msgs.length; i++) {
                    _this.onMessage(msgs[i]);
                }
            }
            else {
                _this.onMessage(msgs);
            }
            // new package arrived, update the heartbeat timeout
            _this.heartbeat.refreshNextTickTimeout();
        };
        socket.onerror = function (event) {
            console.error('[regax-websocket] socket error: ', event && event.error || '');
            _this.emit(SocketEvent.ERROR, common_1.RegaxError.create(event && event.error ? event.error : 'socket error', 'SOCKET_ERROR'), event);
        };
        socket.onclose = function (event) {
            console.info('[regax-websocket] socket close: ', event && event.error || '');
            _this.tryToReconnectIfFailed(url, opts);
        };
        console.log("[regax-websocket] connect to " + url + (this.reconnectAttempts !== 0 ? ": " + (this.reconnectAttempts + 1) + " times" : ''));
        return this;
    };
    RegaxWebSocket.prototype.tryToReconnectIfFailed = function (url, opts) {
        var _this = this;
        if (this.reconnect && this.reconnectAttempts < this.maxReconnectAttempts) {
            this.reconnectAttempts++;
            if (this.reconncetTimer)
                clearTimeout(this.reconncetTimer);
            this.reconncetTimer = setTimeout(function () {
                _this.connect(url, opts);
            }, this.reconnectionDelay);
            this.reconnectionDelay *= 2;
        }
        else {
            this.reconnectReset();
            this.emit(SocketEvent.DISCONNECT);
        }
    };
    RegaxWebSocket.prototype.send = function (buffer) {
        if (this.socket)
            this.socket.send(buffer.buffer);
        return this;
    };
    RegaxWebSocket.prototype.close = function () {
        if (this.isClosed)
            return;
        this.isClosed = true;
        if (this.socket) {
            if (this.socket.close)
                this.socket.close();
            this.socket = undefined;
        }
        this.reconnectReset();
        this.heartbeat.stop();
    };
    RegaxWebSocket.prototype.onMessage = function (msg) {
        var _this = this;
        var type = msg.type, body = msg.body;
        switch (type) {
            case protocol_1.PackageType.HANDSHAKE:
                var data = JSON.parse(protocol_1.strdecode(body));
                if (data.code === ResCode.OLD_CLIENT) {
                    this.emit(SocketEvent.ERROR, common_1.RegaxError.create('client version not fullfill'));
                    return;
                }
                if (data.code !== ResCode.OK) {
                    this.emit(SocketEvent.ERROR, common_1.RegaxError.create(data.message || 'handshake fail', data.code));
                    return;
                }
                if (data.sys && data.sys.heartbeat) {
                    var heartbeatInterval = data.sys.heartbeat; // heartbeat interval
                    var heartbeatTimeout = heartbeatInterval * 2; // max heartbeat timeout
                    this.heartbeat.setTick(heartbeatInterval, heartbeatTimeout);
                }
                else {
                    this.heartbeat.setTick(0, 0);
                }
                this.send(protocol_1.Package.encode(protocol_1.PackageType.HANDSHAKE_ACK));
                // handshake success
                this.emit(SocketEvent.CONNECTION, data);
                break;
            case protocol_1.PackageType.HEARTBEAT:
                this.emit(SocketEvent.HEARTBEAT);
                this.heartbeat.next(function () {
                    var obj = protocol_1.Package.encode(protocol_1.PackageType.HEARTBEAT);
                    _this.send(obj);
                }, function () {
                    console.error('server heartbeat timeout');
                    _this.close();
                });
                break;
            case protocol_1.PackageType.DATA:
                this.emit(SocketEvent.DATA, body);
                break;
            case protocol_1.PackageType.KICK:
                this.emit(SocketEvent.KICK, JSON.parse(protocol_1.strdecode(body)));
                break;
        }
    };
    RegaxWebSocket.prototype.reconnectReset = function () {
        this.reconnect = false;
        this.reconnectionDelay = exports.DEFAULT_RECONNECT_DELAY;
        this.reconnectAttempts = 0;
        clearTimeout(this.reconncetTimer);
    };
    RegaxWebSocket.event = SocketEvent;
    return RegaxWebSocket;
}(common_1.EventEmitter));
exports.RegaxWebSocket = RegaxWebSocket;
//# sourceMappingURL=socket.js.map